from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from cryptography.hazmat.primitives import hashes
from cryptography.fernet import Fernet
import time,json,base64,random
class Crypt:
    LETTERS="ABCDEFGHIJKLMNOPQRSTUVWXYZ9"
    def __init__(self,secret):
        random.seed(secret+"426838")
        self.address = ''.join(random.choice(self.LETTERS) for i in range(81))
        self.f = Fernet(self.genKey(secret))
        
    def encrypt(self,string):
        return self.f.encrypt(bytes(string,'utf-8'))
    
    def decrypt(self,byteString):
        try:
            return str(self.f.decrypt(byteString),'utf-8')
        except TypeError:
            raise Exception("ERROR: Trying to decrypt something that is not encrypted.")

    
    def genKey(self,secret):
        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(),
            length=32,
            salt=bytes(83),
            iterations=100000,
        )
        return base64.urlsafe_b64encode(kdf.derive(bytes(secret,'utf-8')))



import time,json
from iota import Iota, ProposedTransaction, Address, TryteString, Fragment, Transaction,adapter
from .common import Crypt
class Controller(Crypt):
    def __init__(self,secret):
        super(Controller,self).__init__(secret)
        self.api = Iota(adapter='https://nodes.thetangle.org:443',local_pow=False)

    def emit(self,message):
        datum = self.encrypt(json.dumps([time.time(),message]))
        tx = ProposedTransaction(
            address = Address(self.address),
            value = 0,
            message = TryteString.from_bytes(datum)
        )
        startTime = time.time()
        print(f"Sending datum: {datum}")
        try:
            bundle = self.api.send_transfer(transfers=[tx])['bundle']
        except adapter.BadApiResponse as e:
            print("Bad response from server. Failed to send. Retry")
            self.emit(message)
            
        duration = time.time() - startTime
        reference = bundle.transactions[0].hash            
        print(f'Sent in {duration:.01f} secconds to {reference}')

if __name__=="__main__":
    import argparse
    parser = argparse.ArgumentParser(description='Process some integers.')
    parser.add_argument('secret', type=str, help='A passphrade used for encrytion and decryption.')
    parser.add_argument('message', type=str, help='Data you want securly stored.')
    args = parser.parse_args()
    controller = Controller(args.secret)
    controller.emit(args.message)



from .emit import Controller
from .collect import Collector
import argparse
def main():
    parser = argparse.ArgumentParser(description='Process some integers.')
    parser.add_argument('mode', type=str,choices=['emit','read'], help='Either emit or read a message.')
    parser.add_argument('secret', type=str, help='A passphrade used for encrytion and decryption.')
    parser.add_argument('--message', type=str, help='Data you want securly stored. Use in emit mode.',default=None)
    args = parser.parse_args()
    if args.mode == 'emit':
        if args.message is None:
            raise Exception("You forgot your --message argument. See --help for details.")
        controller = Controller(args.secret)
        controller.emit(args.message)
    elif args.mode == 'read':
        collector = Collector(args.secret)
        print(collector.readLatest())




import time,json,math
from iota import Iota, ProposedTransaction, Address, TryteString, Fragment, Transaction
from .common import Crypt
class Collector(Crypt):
    def __init__(self,secret):
        super(Collector,self).__init__(secret)
        self.api = Iota(adapter='https://nodes.thetangle.org:443')

    def getData(self,txHash):
        hashes=api.find_transactions(addresses=[address,])['hashes']
        
    def decode(self,frag):
        hiddenMessage = frag.signature_message_fragment.decode()
        decrypted = self.decrypt(bytes(hiddenMessage,'utf=8'))
        return json.loads(decrypted)
        
    def read(self,silent=False):
        address = Address(self.address)
        txHashes = self.api.find_transactions(addresses=[address,])['hashes']
        batchSize=80
        sets = math.ceil(len(txHashes)/batchSize)
        txs = []
        for i in range(sets):
            start=time.time()
            resp = self.api.get_transaction_objects(txHashes[i*batchSize:(i+1)*batchSize])['transactions']
            txs+=resp
            duration = time.time() - start
            if not silent: print(f'Fetched {len(resp)} events in {duration:.1f} sec.')
        return sorted([self.decode(t) for t in txs], key=lambda a: a[0],reverse=True)
    
    def readLatest(self,silent=False):
        messages = self.read()
        if not silent: print("Latest Message:")
        return messages[0][1]
if __name__=='__main__':
    import argparse
    parser = argparse.ArgumentParser(description='Process some integers.')
    parser.add_argument('secret', type=str, help='A passphrade used for encrytion and decryption.')
    args = parser.parse_args()
    collector = Collector(args.secret)
    print(collector.readLatest())
